/*==========================================================
 * Program : mbchecker.c                   Project : smslink
 * Author  : Philippe Andersson.
 * Date    : 16/05/99
 * Version : 0.02a
 * Notice  : (c) Les Ateliers du Heron, 1998 for Scitex Europe, S.A.
 * Comment : Mailbox checking functions for the smslink server.
 *
 * Modification History :
 * - 0.01a (05/12/98) : Initial release.
 * - 0.02a (16/05/99) : Added include line for errno.h, to solve
 *   compilation problem on RedHat platform.
 *========================================================*/

#include <stdio.h>                         /* for fprintf */
#include <stdlib.h>                  /* for errno & stuff */
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <sys/time.h>           /* for the struct timeval */
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>                        /* semaphores */
#include <sys/shm.h>                     /* shared memory */
#include <sys/ioctl.h>            /* for the ioctl() call */
#include <dial/modems.h>           /* requires 'libmodem' */

#include "sms_serv.h"

/*========================================================*/
/* For debugging purposes only - comment out for normal compile */
/* #define INCL_DEBUG_CODE */

/*-------------------------------------External variables */
/* program specific */

/*---------------------------------------Global variables */
int MBC_instance;
int MBC_instance_is_set = FALSE;

/*========================================================*/
void MBC_unlock_gsm ()
{
  extern int shmem_sem;
  extern struct gsms_def *gsmdevices;

  syslog ((FACILITY | LOG_WARNING), "MBC being hit by SIGTERM.");
  /* set the allocated GSM module back to 'free' status */
  if (MBC_instance_is_set) {
    if (sem_wait (shmem_sem) == -1)
      syserr ("sms_serv: failed to wait on shared mem. semaphore");

    /* ---- Begin Crit. Sect. #5 ---- */
    if (!gsmdevices[MBC_instance].free) {
      gsmdevices[MBC_instance].free = TRUE;
      gsmdevices[MBC_instance].owner = 0;
      syslog ((FACILITY | LOG_WARNING), "GSM instance </dev/%s> has been unlocked.",
             gsmdevices[MBC_instance].device);
    }
    /* leave crit. sect. */
    if (sem_signal (shmem_sem) == -1)
      syserr ("sms_serv: can't signal shared mem. semaphore");
    /* ---- End Crit. Sect. #5 ---- */
  }
  /* free what's need to be freed and exit */
  /*------------------------------------------------------*/
  exit (0);
}                                    /* MBC_unlock_gsm () */
/*========================================================*/
int all_done (int *array, int size)
{
  int retval = TRUE;
  int i;

  for (i = 0; i < size; i++) {
    retval = (retval && array[i]);
  }
  return (retval);
}                                          /* all_done () */
/*========================================================*/
int mbcheck (struct gsms_def *gsm)
{
  return (0);
}                                           /* mbcheck () */
/*========================================================*/
void mbcheck_wrapper ()
{
  extern int errno;
  extern int global_sem;
  extern int shmem_sem;
  extern struct gsms_def *gsmdevices;
  extern int ngsmdevs;     /* num. configured GSM devices */
  int nwaitsecs = 0;
  int last_errno;
  int retval;
  char device[MAXDEVLEN];
  struct gsms_def gsm;
  int msgin;
  int i;
  int gsm_done[ngsmdevs];
  int maxiterat;
  int iteration;
  char *missedg;
  struct sigaction sa_c;

  /*--------------------------------------Initializations */
  for (i = 0; i < ngsmdevs; i++) {
    gsm_done[i] = FALSE;
  }
  maxiterat = (2 * ngsmdevs);
  iteration = 0;
  
  /*--------------------------------------------Main loop */
  while ((!all_done (gsm_done, ngsmdevs)) && (iteration < maxiterat)) {
    syslog ((FACILITY | LOG_INFO), "MBC waits for a free GSM instance (pass %d)...",
           (iteration + 1));
  
    while (((retval = sem_decreq (global_sem)) == -1) &&
          (errno == EAGAIN) &&
	  (nwaitsecs < M_TIMEOUT)) {
      sleep (W_STEP);
      nwaitsecs += W_STEP;
    }                                      /* while (...) */
    last_errno = errno;
  
    if (retval == -1) {          /* failed to get a modem */
      switch (last_errno) {
        case EAGAIN: {
          syslog ((FACILITY | LOG_WARNING), "MBC timeout expired (all GSMs busy).");
          break;
        }

        default: {
          syserr ("sms_serv(MBC): can't decrease global semaphore");
	  break;
        }
      }                            /* switch (last_errno) */
    }
    else {               /* ---- Begin Crit. Sect. #1 --- */
      /* at least 1 GSM module is free - find it */
      if (sem_wait (shmem_sem) == -1)
        syserr ("sms_serv(MBC): failed to wait on shared mem. semaphore");

      /* ---- Begin Crit. Sect. #2 ---- */
      /* first catch SIGTERM to handle child being killed while GSM's locked */
      sa_c.sa_handler = MBC_unlock_gsm;
      if (sigfillset (&sa_c.sa_mask) == -1)
        syserr ("sms_serv(MBC): can't fill signal set");
      sa_c.sa_flags = 0;
      if (sigaction (SIGTERM, &sa_c, NULL) == -1)
        syserr ("sms_serv(MBC): can't catch SIGTERM");

      /* search for MBC_instance both free and not yet processed */
      MBC_instance = 0;
      while (MBC_instance < ngsmdevs) {
	if (gsmdevices[MBC_instance].free && !gsm_done[MBC_instance]) {
	  /* let's process this one */
	  break;
	}
        MBC_instance++;
      }                     /* while (MBC_instance < ngsmdevs) */
      if (MBC_instance < ngsmdevs) {
        /* got it - now lock it (device lockfile handled by libmodem) */
        MBC_instance_is_set = TRUE;
        gsmdevices[MBC_instance].free = FALSE;
        gsmdevices[MBC_instance].owner = getpid ();
      
        /* copy to "local" space, to avoid the shmem balagan */
        if (gsmdevcpy (&gsm, &gsmdevices[MBC_instance]) == -1)
          syserr ("sms_serv(MBC): error copying GSM MBC_instance");
      
        /* leave crit. sect. */
        if (sem_signal (shmem_sem) == -1)
          syserr ("sms_serv(MBC): can't signal shared mem. semaphore");
        /* ---- End Crit. Sect. #2 ---- */

        /* log it */
        syslog ((FACILITY | LOG_NOTICE), "MBC now polling device </dev/%s>.", gsm.device);
    
        /* ---->>> Actual GSM Dialogue <<<---- */
        if ((msgin = mbcheck (&gsm)) != -1) {
          syslog ((FACILITY | LOG_NOTICE), "MBC check ok, got %d msg from </dev/%s>.", msgin, gsm.device);
        }
        else { /* some error occured */
          syslog ((FACILITY | LOG_WARNING), "MBC ERROR: failed to poll </dev/%s>.", gsm.device);
        }

	/* mark this GSM as "done" */
	gsm_done[MBC_instance] = TRUE;
    
        /* now free the modem again */
        if (sem_wait (shmem_sem) == -1)
          syserr ("sms_serv(MBC): failed to wait on shared mem. semaphore");

        /* ---- Begin Crit. Sect. #3 ---- */
        gsmdevices[MBC_instance].free = TRUE;
        gsmdevices[MBC_instance].owner = 0;
        MBC_instance_is_set = FALSE;
        /* leave crit. sect. */
        if (sem_signal (shmem_sem) == -1)
          syserr ("sms_serv(MBC): can't signal shared mem. semaphore");
        /* ---- End Crit. Sect. #3 ---- */

        /* leave main crit. sect. */
        if (sem_signal (global_sem) == -1)
          syserr ("sms_serv(MBC): can't signal global semaphore");
	/* ---- End Crit. Sect. #1 ---- */
      }
      else {
        /* leave crit. sect. */
        if (sem_signal (shmem_sem) == -1)
          syserr ("sms_serv(MBC): can't signal shared mem. semaphore");
        /* ---- End Crit. Sect. #2 ---- */

        /* leave main crit. sect. - release unused resource */
        if (sem_signal (global_sem) == -1)
          syserr ("sms_serv(MBC): can't signal global semaphore");
	/* ---- End Crit. Sect. #1 ---- */

        syslog ((FACILITY | LOG_WARNING), "MBC all free GSM's already processed");
      }                   /* if (MBC_instance < ngsmdevs) */
    }                                /* if (retval == -1) */
    iteration++;
  }                               /* while ((!all_done... */
  /*-------------------------------------------Conclusion */
  /* Based on gsm_done, report on missed MBC_instances */
  if (all_done (gsm_done, ngsmdevs)) {
    /* all done */
    syslog ((FACILITY | LOG_NOTICE), "MBC all devices processed.");
  }
  else {
    /* missed some */
    missedg = (char *) malloc ((MINIBUFF + 1) * sizeof (char));
    missedg[0] = '\0';
    for (i = 0; i < ngsmdevs; i++) {
      if (!gsm_done[i]) {
        strcat (missedg, gsmdevices[i].device);
	strcat (missedg, ", ");
      }
    }
    /* remove trailing ", " sequence */
    missedg[strlen (missedg) - 2] = '\0';
    syslog ((FACILITY | LOG_WARNING), "MBC poll missed {%s}.", missedg);
    free (missedg);
  }
}                                   /* mbcheck_wrapper () */
/*========================================================*/

/*==========================================================
 * EOF : mbchecker.c
 *===================*/
