/*==========================================================
 * Program : sms_serv.h                    Project : smslink
 * Author  : Philippe Andersson.
 * Date    : 14/02/99
 * Version : 0.19b
 * Notice  : (c) Les Ateliers du Heron, 1998 for Scitex Europe, S.A.
 * Comment : Main include file for SMSLink server.
 *
 * Modification History :
 * - 0.01a (13/08/98) : Initial release.
 * - 0.02a (25/08/98) : Added section for logging facility and
 *   a function declarations section. Cosmetics.
 * - 0.10a (27/08/98) : Integration of the parser module into the
 *   main sms_serv program. Merged server.h into this file.
 *   Added code to prevent double inclusion. Cosmetics.
 * - 0.11a (01/09/98) : Increased BUFFSIZE from 200 to 400.
 * - 0.12a (27/09/98) : Added MINIBUFF define.
 * - 1.13a (19/10/98) : Added GSMDEVFILE define and function
 *   definitions for gsmdevices.c. Changed 'struct modem_def'
 *   to 'struct gsms_def' and added fields.
 * ++++ Switched to Beta ++++
 * - 0.14b (23/10/98) : Added 'user' member in 'symbols' struct
 *   and defined MAXUIDLEN.
 * - 0.15b (22/11/98) : Added MBCHKINTERVAL define.
 * - 0.16b (13/12/98) : Added member "owner" to struct gsms_def.
 * - 0.17b (06/02/99) : Added support for access control.
 *   Contributed by Philipp Klaus <pklaus@access.ch>.
 * - 0.18b (11/02/99) : Start rewriting of ACL support.
 * - 0.19b (14/02/99) : Declared public functions for new ACL
 *   implementation.
 *========================================================*/

#ifndef _SMS_SERV_H
#define _SMS_SERV_H

/* Program information */
#define SMS_SERV_VERSION	"0.40b"
#define SMS_SERV_DATE		"16/05/99"
#define PROMPT			"SMS> "

/* GSM devices */
#define GSMDEVFILE		"/etc/gsmdevices"
#define PINLEN			4
#define PUKLEN			8
#define MAXATTEMPTS		4        	/* 3 wrong PIN's + 1 PUK */
#define CTRL_Z			"\x1A"		/* SM end marker */

/* access control (INA) */
#define ACCESSFILE		"/etc/gsmaccess"
#define MAXACLS			10000		/* precaution */
#define ACL_ALLOW		1
#define ACL_DENY		2

/* Buffer handling */
#define BUFFSIZE		400
#define MINIBUFF		80		/* for hostname, for inst. */
#define BIGBUFF			1024

/* SMS Messages-related */
#define MAXPHNUMLEN		80		/* max length for phone num */
#define MAXMSGLEN		160		/* max msg length for SMS */
#define DEFAULTSMSC		"+3275161616"	/* Proximus OK */
#define MAXUIDLEN		12		/* max length for sender ID */

/* For incoming messages handling */
#define MBCHKINTERVAL		3600		/* mailbox check interv.(sec)*/
/* #define MBCHKINTERVAL		600 */

/* For the logging calls */
#include <syslog.h>
#define FACILITY		LOG_USER	/* default */

/* Local socket options */
#define MAXCLIENTCONN		5		/* max socket queue length */
#define MAXCHILDPROC		5		/* max child processes */

/* Semaphore-related parameters */
#define MAXMODEMS		2		/* max supported modem devs */
#define M_TIMEOUT		90		/* secs wait for free modem */
#define W_STEP			5		/* secs to sleep */

/* Modem-related parameters */
#define MAXDEVLEN		40		/* max. device name length */

/* Can be usefull */
#define TRUE        1
#define FALSE       0

/*==========================================================
 * Structure Declarations
 *========================================================*/
#include <unistd.h>                  /* for pid_t typedef */
#include <netinet/in.h>            /* for struct acl_item */

/* This is the info structure defining an SMS message */
struct symbols {
  char *smsc;                           /* message center */
  char *destgsm;                       /* destination gsm */
  char *message;                          /* message text */
  char *user;                    /* sender identification */
};

/* This defines a gsm device instance */
struct gsms_def {
  int free;                                 /* bool value */
  pid_t owner;                    /* pid of owner process */
  char device[MAXDEVLEN + 1];              /* device name */
  char PIN[PINLEN + 1];                       /* PIN code */
  char PUK[PUKLEN + 1];                       /* PUK code */
  char addr[MAXPHNUMLEN + 1];       /* SIM calling number */
  char defsca[MAXPHNUMLEN + 1]; /* def. Service Center Addr. */
  char provider[MAXDEVLEN + 1]; /* GSM Net. provider name */
};

/* structures for access control (ACL items) */
struct acl_item {
  int action;                          /* allow or reject */
  struct in_addr network;                /* network or IP */
  unsigned long nomask;                /* net. order mask */
  struct acl_item *next;
  struct acl_item *prev;
};

typedef struct acl_list_header {
  struct acl_item *head;
  struct acl_item *tail;
} acl_list;

/*==========================================================
 * Function Declarations
 *========================================================*/
/* --- in "stuff.c" --- */
void daemons_death ();              /* main exit function */
void death_of_a_child ();  /* accounting when child exits */
void tellsock (int, char *);
int gsmdevcpy (struct gsms_def *, struct gsms_def *);
int tell_gsm (int, char *);
int get_gsm_answer (int, char *, int, int);
int set_semvalue (int, int);
int sem_wait (int);
int sem_decreq (int);
int sem_signal (int);
void syserr (char *);
void fatal (char *);

/* --- in "mbchecker.c" --- */
void mbcheck_wrapper (void);

/* --- in "accessctrl.c" --- */
void acl_list_init (acl_list *);
void free_acl_list (acl_list *);
int read_acl (acl_list *);
int check_acl (struct in_addr *, acl_list);

/* --- in "gsmdevices.c" --- */
static int open_gsmdevs (void);
static void close_gsmdevs (void);
struct gsms_def *getgsmdevbynam (char *);
int getgsmdevscount (int);
void setgsmdevs (void);
struct gsms_def *getnextgsmdev (void);
static int demangle_gsmdev_entry (struct gsms_def *, char *);

/* --- in "server.y" --- */
int parser ();                           /* parser module */
int my_yyinput (char *, int);       /* new input for flex */

/* --- in "server.l" --- */
void yyerror (const char *);

#endif                             /* #ifndef _SMS_SERV_H */

/*==========================================================
 * EOF : sms_serv.h
 *===================*/
